# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.terminal_action import TerminalActionParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_terminal_action_response import CreateTerminalActionResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.terminal_action_query import TerminalActionQueryParams
from ...types.search_terminal_actions_response import SearchTerminalActionsResponse
from ...types.get_terminal_action_response import GetTerminalActionResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...types.cancel_terminal_action_response import CancelTerminalActionResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawActionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        idempotency_key: str,
        action: TerminalActionParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateTerminalActionResponse]:
        """
        Creates a Terminal action request and sends it to the specified device.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateAction` request. Keys can be any valid string
            but must be unique for every `CreateAction` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more
            information.

        action : TerminalActionParams
            The Action to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateTerminalActionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/terminals/actions",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "action": convert_and_respect_annotation_metadata(
                    object_=action, annotation=TerminalActionParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateTerminalActionResponse,
                    construct_type(
                        type_=CreateTerminalActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        query: typing.Optional[TerminalActionQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchTerminalActionsResponse]:
        """
        Retrieves a filtered list of Terminal action requests created by the account making the request. Terminal action requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalActionQueryParams]
            Queries terminal actions based on given conditions and sort order.
            Leaving this unset will return all actions with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more
            information.

        limit : typing.Optional[int]
            Limit the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchTerminalActionsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/terminals/actions/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=TerminalActionQueryParams, direction="write"
                ),
                "cursor": cursor,
                "limit": limit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchTerminalActionsResponse,
                    construct_type(
                        type_=SearchTerminalActionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetTerminalActionResponse]:
        """
        Retrieves a Terminal action request by `action_id`. Terminal action requests are available for 30 days.

        Parameters
        ----------
        action_id : str
            Unique ID for the desired `TerminalAction`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetTerminalActionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/actions/{jsonable_encoder(action_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTerminalActionResponse,
                    construct_type(
                        type_=GetTerminalActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CancelTerminalActionResponse]:
        """
        Cancels a Terminal action request if the status of the request permits it.

        Parameters
        ----------
        action_id : str
            Unique ID for the desired `TerminalAction`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CancelTerminalActionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/actions/{jsonable_encoder(action_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelTerminalActionResponse,
                    construct_type(
                        type_=CancelTerminalActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawActionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        idempotency_key: str,
        action: TerminalActionParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateTerminalActionResponse]:
        """
        Creates a Terminal action request and sends it to the specified device.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateAction` request. Keys can be any valid string
            but must be unique for every `CreateAction` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more
            information.

        action : TerminalActionParams
            The Action to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateTerminalActionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/terminals/actions",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "action": convert_and_respect_annotation_metadata(
                    object_=action, annotation=TerminalActionParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateTerminalActionResponse,
                    construct_type(
                        type_=CreateTerminalActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        query: typing.Optional[TerminalActionQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchTerminalActionsResponse]:
        """
        Retrieves a filtered list of Terminal action requests created by the account making the request. Terminal action requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalActionQueryParams]
            Queries terminal actions based on given conditions and sort order.
            Leaving this unset will return all actions with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more
            information.

        limit : typing.Optional[int]
            Limit the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchTerminalActionsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/terminals/actions/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=TerminalActionQueryParams, direction="write"
                ),
                "cursor": cursor,
                "limit": limit,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchTerminalActionsResponse,
                    construct_type(
                        type_=SearchTerminalActionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetTerminalActionResponse]:
        """
        Retrieves a Terminal action request by `action_id`. Terminal action requests are available for 30 days.

        Parameters
        ----------
        action_id : str
            Unique ID for the desired `TerminalAction`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetTerminalActionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/actions/{jsonable_encoder(action_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTerminalActionResponse,
                    construct_type(
                        type_=GetTerminalActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CancelTerminalActionResponse]:
        """
        Cancels a Terminal action request if the status of the request permits it.

        Parameters
        ----------
        action_id : str
            Unique ID for the desired `TerminalAction`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CancelTerminalActionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/actions/{jsonable_encoder(action_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelTerminalActionResponse,
                    construct_type(
                        type_=CancelTerminalActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams
from .modifier_location_overrides import ModifierLocationOverridesParams


class CatalogModifierParams(typing_extensions.TypedDict):
    """
    A modifier that can be applied to items at the time of sale. For example, a cheese modifier for a burger, or a flavor modifier for a serving of ice cream.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The modifier name.  This is a searchable attribute for use in applicable query filters, and its value length is of Unicode code points.
    """

    price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The modifier price.
    """

    on_by_default: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    When `true`, this modifier is selected by default when displaying the modifier list.
    This setting can be overridden at the item level using `CatalogModifierListInfo.modifier_overrides`.
    """

    ordinal: typing_extensions.NotRequired[typing.Optional[int]]
    """
    Determines where this `CatalogModifier` appears in the `CatalogModifierList`.
    """

    modifier_list_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the `CatalogModifierList` associated with this modifier.
    """

    location_overrides: typing_extensions.NotRequired[typing.Optional[typing.Sequence[ModifierLocationOverridesParams]]]
    """
    Location-specific price overrides.
    """

    image_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the image associated with this `CatalogModifier` instance.
    Currently this image is not displayed by Square, but is free to be displayed in 3rd party applications.
    """

    hidden_online: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    When `true`, this modifier is hidden from online ordering channels. This setting can be overridden at the item level using `CatalogModifierListInfo.modifier_overrides`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class ExternalPaymentDetailsParams(typing_extensions.TypedDict):
    """
    Stores details about an external payment. Contains only non-confidential information.
    For more information, see
    [Take External Payments](https://developer.squareup.com/docs/payments-api/take-payments/external-payments).
    """

    type: str
    """
    The type of external payment the seller received. It can be one of the following:
    - CHECK - Paid using a physical check.
    - BANK_TRANSFER - Paid using external bank transfer.
    - OTHER\_GIFT\_CARD - Paid using a non-Square gift card.
    - CRYPTO - Paid using a crypto currency.
    - SQUARE_CASH - Paid using Square Cash App.
    - SOCIAL - Paid using peer-to-peer payment applications.
    - EXTERNAL - A third-party application gathered this payment outside of Square.
    - EMONEY - Paid using an E-money provider.
    - CARD - A credit or debit card that Square does not support.
    - STORED_BALANCE - Use for house accounts, store credit, and so forth.
    - FOOD_VOUCHER - Restaurant voucher provided by employers to employees to pay for meals
    - OTHER - A type not listed here.
    """

    source: str
    """
    A description of the external payment source. For example, 
    "Food Delivery Service".
    """

    source_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    An ID to associate the payment to its originating source.
    """

    source_fee_money: typing_extensions.NotRequired[MoneyParams]
    """
    The fees paid to the source. The `amount_money` minus this field is 
    the net amount seller receives.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.workweek_config import WorkweekConfigParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.update_workweek_config_response import UpdateWorkweekConfigResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawWorkweekConfigsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, id: str, *, workweek_config: WorkweekConfigParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateWorkweekConfigResponse]:
        """
        Updates a `WorkweekConfig`.

        Parameters
        ----------
        id : str
            The UUID for the `WorkweekConfig` object being updated.

        workweek_config : WorkweekConfigParams
            The updated `WorkweekConfig` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateWorkweekConfigResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/workweek-configs/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "workweek_config": convert_and_respect_annotation_metadata(
                    object_=workweek_config, annotation=WorkweekConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateWorkweekConfigResponse,
                    construct_type(
                        type_=UpdateWorkweekConfigResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawWorkweekConfigsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, id: str, *, workweek_config: WorkweekConfigParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateWorkweekConfigResponse]:
        """
        Updates a `WorkweekConfig`.

        Parameters
        ----------
        id : str
            The UUID for the `WorkweekConfig` object being updated.

        workweek_config : WorkweekConfigParams
            The updated `WorkweekConfig` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateWorkweekConfigResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/workweek-configs/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "workweek_config": convert_and_respect_annotation_metadata(
                    object_=workweek_config, annotation=WorkweekConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateWorkweekConfigResponse,
                    construct_type(
                        type_=UpdateWorkweekConfigResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogItemProductType = typing.Union[
    typing.Literal[
        "REGULAR",
        "GIFT_CARD",
        "APPOINTMENTS_SERVICE",
        "FOOD_AND_BEV",
        "EVENT",
        "DIGITAL",
        "DONATION",
        "LEGACY_SQUARE_ONLINE_SERVICE",
        "LEGACY_SQUARE_ONLINE_MEMBERSHIP",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .wage_setting import WageSettingParams
import typing
from .error import ErrorParams


class GetWageSettingResponseParams(typing_extensions.TypedDict):
    """
    Represents a response from a retrieve request containing the specified `WageSetting` object or error messages.
    """

    wage_setting: typing_extensions.NotRequired[WageSettingParams]
    """
    The successfully retrieved `WageSetting` object.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    The errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .select_option import SelectOption
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SelectOptions(UncheckedBaseModel):
    title: str = pydantic.Field()
    """
    The title text to display in the select flow on the Terminal.
    """

    body: str = pydantic.Field()
    """
    The body text to display in the select flow on the Terminal.
    """

    options: typing.List[SelectOption] = pydantic.Field()
    """
    Represents the buttons/options that should be displayed in the select flow on the Terminal.
    """

    selected_option: typing.Optional[SelectOption] = pydantic.Field(default=None)
    """
    The buyer’s selected option.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogTimePeriod(UncheckedBaseModel):
    """
    Represents a time period - either a single period or a repeating period.
    """

    event: typing.Optional[str] = pydantic.Field(default=None)
    """
    An iCalendar (RFC 5545) [event](https://tools.ietf.org/html/rfc5545#section-3.6.1), which
    specifies the name, timing, duration and recurrence of this time period.
    
    Example:
    
    ```
    DTSTART:20190707T180000
    DURATION:P2H
    RRULE:FREQ=WEEKLY;BYDAY=MO,WE,FR
    ```
    
    Only `SUMMARY`, `DTSTART`, `DURATION` and `RRULE` fields are supported.
    `DTSTART` must be in local (unzoned) time format. Note that while `BEGIN:VEVENT`
    and `END:VEVENT` is not required in the request. The response will always
    include them.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .event_data import EventDataParams


class EventParams(typing_extensions.TypedDict):
    merchant_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the target merchant associated with the event.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the target location associated with the event.
    """

    type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The type of event this represents.
    """

    event_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID for the event.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    Timestamp of when the event was created, in RFC 3339 format.
    """

    data: typing_extensions.NotRequired[EventDataParams]
    """
    The data associated with the event.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .catalog_object import CatalogObjectParams


class BatchGetCatalogObjectsResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    objects: typing_extensions.NotRequired[typing.Sequence[CatalogObjectParams]]
    """
    A list of [CatalogObject](entity:CatalogObject)s returned.
    """

    related_objects: typing_extensions.NotRequired[typing.Sequence[CatalogObjectParams]]
    """
    A list of [CatalogObject](entity:CatalogObject)s referenced by the object in the `objects` field.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LoyaltyEventOtherParams(typing_extensions.TypedDict):
    """
    Provides metadata when the event `type` is `OTHER`.
    """

    loyalty_program_id: str
    """
    The Square-assigned ID of the [loyalty program](entity:LoyaltyProgram).
    """

    points: int
    """
    The number of points added or removed.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

DisputeReason = typing.Union[
    typing.Literal[
        "AMOUNT_DIFFERS",
        "CANCELLED",
        "DUPLICATE",
        "NO_KNOWLEDGE",
        "NOT_AS_DESCRIBED",
        "NOT_RECEIVED",
        "PAID_BY_OTHER_MEANS",
        "CUSTOMER_REQUESTS_CREDIT",
        "EMV_LIABILITY_SHIFT",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

GiftCardActivityAdjustDecrementReason = typing.Union[
    typing.Literal["SUSPICIOUS_ACTIVITY", "BALANCE_ACCIDENTALLY_INCREASED", "SUPPORT_ISSUE", "PURCHASE_WAS_REFUNDED"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ....core.client_wrapper import SyncClientWrapper
from ....requests.loyalty_promotion import LoyaltyPromotionParams
from ....core.request_options import RequestOptions
from ....core.http_response import HttpResponse
from ....types.create_loyalty_promotion_response import CreateLoyaltyPromotionResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.serialization import convert_and_respect_annotation_metadata
from ....core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ....core.api_error import ApiError
from ....types.get_loyalty_promotion_response import GetLoyaltyPromotionResponse
from ....types.cancel_loyalty_promotion_response import CancelLoyaltyPromotionResponse
from ....core.client_wrapper import AsyncClientWrapper
from ....core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPromotionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        program_id: str,
        *,
        loyalty_promotion: LoyaltyPromotionParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateLoyaltyPromotionResponse]:
        """
        Creates a loyalty promotion for a [loyalty program](entity:LoyaltyProgram). A loyalty promotion
        enables buyers to earn points in addition to those earned from the base loyalty program.

        This endpoint sets the loyalty promotion to the `ACTIVE` or `SCHEDULED` status, depending on the
        `available_time` setting. A loyalty program can have a maximum of 10 loyalty promotions with an
        `ACTIVE` or `SCHEDULED` status.

        Parameters
        ----------
        program_id : str
            The ID of the [loyalty program](entity:LoyaltyProgram) to associate with the promotion.
            To get the program ID, call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram)
            using the `main` keyword.

        loyalty_promotion : LoyaltyPromotionParams
            The loyalty promotion to create.

        idempotency_key : str
            A unique identifier for this request, which is used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateLoyaltyPromotionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/loyalty/programs/{jsonable_encoder(program_id)}/promotions",
            method="POST",
            json={
                "loyalty_promotion": convert_and_respect_annotation_metadata(
                    object_=loyalty_promotion, annotation=LoyaltyPromotionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLoyaltyPromotionResponse,
                    construct_type(
                        type_=CreateLoyaltyPromotionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, promotion_id: str, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetLoyaltyPromotionResponse]:
        """
        Retrieves a loyalty promotion.

        Parameters
        ----------
        promotion_id : str
            The ID of the [loyalty promotion](entity:LoyaltyPromotion) to retrieve.

        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram). To get the program ID,
            call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram) using the `main` keyword.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetLoyaltyPromotionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/loyalty/programs/{jsonable_encoder(program_id)}/promotions/{jsonable_encoder(promotion_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLoyaltyPromotionResponse,
                    construct_type(
                        type_=GetLoyaltyPromotionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(
        self, promotion_id: str, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CancelLoyaltyPromotionResponse]:
        """
        Cancels a loyalty promotion. Use this endpoint to cancel an `ACTIVE` promotion earlier than the
        end date, cancel an `ACTIVE` promotion when an end date is not specified, or cancel a `SCHEDULED` promotion.
        Because updating a promotion is not supported, you can also use this endpoint to cancel a promotion before
        you create a new one.

        This endpoint sets the loyalty promotion to the `CANCELED` state

        Parameters
        ----------
        promotion_id : str
            The ID of the [loyalty promotion](entity:LoyaltyPromotion) to cancel. You can cancel a
            promotion that has an `ACTIVE` or `SCHEDULED` status.

        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CancelLoyaltyPromotionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/loyalty/programs/{jsonable_encoder(program_id)}/promotions/{jsonable_encoder(promotion_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelLoyaltyPromotionResponse,
                    construct_type(
                        type_=CancelLoyaltyPromotionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawPromotionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        program_id: str,
        *,
        loyalty_promotion: LoyaltyPromotionParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateLoyaltyPromotionResponse]:
        """
        Creates a loyalty promotion for a [loyalty program](entity:LoyaltyProgram). A loyalty promotion
        enables buyers to earn points in addition to those earned from the base loyalty program.

        This endpoint sets the loyalty promotion to the `ACTIVE` or `SCHEDULED` status, depending on the
        `available_time` setting. A loyalty program can have a maximum of 10 loyalty promotions with an
        `ACTIVE` or `SCHEDULED` status.

        Parameters
        ----------
        program_id : str
            The ID of the [loyalty program](entity:LoyaltyProgram) to associate with the promotion.
            To get the program ID, call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram)
            using the `main` keyword.

        loyalty_promotion : LoyaltyPromotionParams
            The loyalty promotion to create.

        idempotency_key : str
            A unique identifier for this request, which is used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateLoyaltyPromotionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/loyalty/programs/{jsonable_encoder(program_id)}/promotions",
            method="POST",
            json={
                "loyalty_promotion": convert_and_respect_annotation_metadata(
                    object_=loyalty_promotion, annotation=LoyaltyPromotionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateLoyaltyPromotionResponse,
                    construct_type(
                        type_=CreateLoyaltyPromotionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, promotion_id: str, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetLoyaltyPromotionResponse]:
        """
        Retrieves a loyalty promotion.

        Parameters
        ----------
        promotion_id : str
            The ID of the [loyalty promotion](entity:LoyaltyPromotion) to retrieve.

        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram). To get the program ID,
            call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram) using the `main` keyword.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetLoyaltyPromotionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/loyalty/programs/{jsonable_encoder(program_id)}/promotions/{jsonable_encoder(promotion_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetLoyaltyPromotionResponse,
                    construct_type(
                        type_=GetLoyaltyPromotionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(
        self, promotion_id: str, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CancelLoyaltyPromotionResponse]:
        """
        Cancels a loyalty promotion. Use this endpoint to cancel an `ACTIVE` promotion earlier than the
        end date, cancel an `ACTIVE` promotion when an end date is not specified, or cancel a `SCHEDULED` promotion.
        Because updating a promotion is not supported, you can also use this endpoint to cancel a promotion before
        you create a new one.

        This endpoint sets the loyalty promotion to the `CANCELED` state

        Parameters
        ----------
        promotion_id : str
            The ID of the [loyalty promotion](entity:LoyaltyPromotion) to cancel. You can cancel a
            promotion that has an `ACTIVE` or `SCHEDULED` status.

        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CancelLoyaltyPromotionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/loyalty/programs/{jsonable_encoder(program_id)}/promotions/{jsonable_encoder(promotion_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelLoyaltyPromotionResponse,
                    construct_type(
                        type_=CancelLoyaltyPromotionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .payout_status import PayoutStatus
from .money import Money
from .destination import Destination
from .payout_type import PayoutType
from .payout_fee import PayoutFee
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Payout(UncheckedBaseModel):
    """
    An accounting of the amount owed the seller and record of the actual transfer to their
    external bank account or to the Square balance.
    """

    id: str = pydantic.Field()
    """
    A unique ID for the payout.
    """

    status: typing.Optional[PayoutStatus] = pydantic.Field(default=None)
    """
    Indicates the payout status.
    See [PayoutStatus](#type-payoutstatus) for possible values
    """

    location_id: str = pydantic.Field()
    """
    The ID of the location associated with the payout.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the payout was created and submitted for deposit to the seller's banking destination, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the payout was last updated, in RFC 3339 format.
    """

    amount_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money involved in the payout. A positive amount indicates a deposit, and a negative amount indicates a withdrawal. This amount is never zero.
    """

    destination: typing.Optional[Destination] = pydantic.Field(default=None)
    """
    Information about the banking destination (such as a bank account, Square checking account, or debit card)
    against which the payout was made.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version number, which is incremented each time an update is made to this payout record.
    The version number helps developers receive event notifications or feeds out of order.
    """

    type: typing.Optional[PayoutType] = pydantic.Field(default=None)
    """
    Indicates the payout type.
    See [PayoutType](#type-payouttype) for possible values
    """

    payout_fee: typing.Optional[typing.List[PayoutFee]] = pydantic.Field(default=None)
    """
    A list of transfer fees and any taxes on the fees assessed by Square for this payout.
    """

    arrival_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The calendar date, in ISO 8601 format (YYYY-MM-DD), when the payout is due to arrive in the seller’s banking destination.
    """

    end_to_end_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID for each `Payout` object that might also appear on the seller’s bank statement. You can use this ID to automate the process of reconciling each payout with the corresponding line item on the bank statement.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawCardsClient
from ..types.sort_order import SortOrder
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.card import Card
from ..types.list_cards_response import ListCardsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.card import CardParams
from ..types.create_card_response import CreateCardResponse
from ..types.get_card_response import GetCardResponse
from ..types.disable_card_response import DisableCardResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCardsClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCardsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        customer_id: typing.Optional[str] = None,
        include_disabled: typing.Optional[bool] = None,
        reference_id: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Card]:
        """
        Retrieves a list of cards owned by the account making the request.
        A max of 25 cards will be returned.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        customer_id : typing.Optional[str]
            Limit results to cards associated with the customer supplied.
            By default, all cards owned by the merchant are returned.

        include_disabled : typing.Optional[bool]
            Includes disabled cards.
            By default, all enabled cards owned by the merchant are returned.

        reference_id : typing.Optional[str]
            Limit results to cards associated with the reference_id supplied.

        sort_order : typing.Optional[SortOrder]
            Sorts the returned list by when the card was created with the specified order.
            This field defaults to ASC.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Card]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.cards.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/cards",
            method="GET",
            params={
                "cursor": cursor,
                "customer_id": customer_id,
                "include_disabled": include_disabled,
                "reference_id": reference_id,
                "sort_order": sort_order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCardsResponse,
                    construct_type(
                        type_=ListCardsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    customer_id=customer_id,
                    include_disabled=include_disabled,
                    reference_id=reference_id,
                    sort_order=sort_order,
                    request_options=request_options,
                )
                _items = _parsed_response.cards
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        idempotency_key: str,
        source_id: str,
        card: CardParams,
        verification_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCardResponse:
        """
        Adds a card on file to an existing merchant.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this CreateCard request. Keys can be
            any valid string and must be unique for every request.

            Max: 45 characters

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        source_id : str
            The ID of the source which represents the card information to be stored. This can be a card nonce or a payment id.

        card : CardParams
            Payment details associated with the card to be stored.

        verification_token : typing.Optional[str]
            An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

            See the [SCA Overview](https://developer.squareup.com/docs/sca-overview).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.cards.create(
            idempotency_key="4935a656-a929-4792-b97c-8848be85c27c",
            source_id="cnon:uIbfJXhXETSP197M3GB",
            card={
                "cardholder_name": "Amelia Earhart",
                "billing_address": {
                    "address_line1": "500 Electric Ave",
                    "address_line2": "Suite 600",
                    "locality": "New York",
                    "administrative_district_level1": "NY",
                    "postal_code": "10003",
                    "country": "US",
                },
                "customer_id": "VDKXEEKPJN48QDG3BGGFAK05P8",
                "reference_id": "user-id-1",
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key,
            source_id=source_id,
            card=card,
            verification_token=verification_token,
            request_options=request_options,
        )
        return response.data

    def get(self, card_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetCardResponse:
        """
        Retrieves details for a specific Card.

        Parameters
        ----------
        card_id : str
            Unique ID for the desired Card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.cards.get(
            card_id="card_id",
        )
        """
        response = self._raw_client.get(card_id, request_options=request_options)
        return response.data

    def disable(self, card_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DisableCardResponse:
        """
        Disables the card, preventing any further updates or charges.
        Disabling an already disabled card is allowed but has no effect.

        Parameters
        ----------
        card_id : str
            Unique ID for the desired Card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DisableCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.cards.disable(
            card_id="card_id",
        )
        """
        response = self._raw_client.disable(card_id, request_options=request_options)
        return response.data


class AsyncCardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCardsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCardsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        customer_id: typing.Optional[str] = None,
        include_disabled: typing.Optional[bool] = None,
        reference_id: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Card]:
        """
        Retrieves a list of cards owned by the account making the request.
        A max of 25 cards will be returned.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        customer_id : typing.Optional[str]
            Limit results to cards associated with the customer supplied.
            By default, all cards owned by the merchant are returned.

        include_disabled : typing.Optional[bool]
            Includes disabled cards.
            By default, all enabled cards owned by the merchant are returned.

        reference_id : typing.Optional[str]
            Limit results to cards associated with the reference_id supplied.

        sort_order : typing.Optional[SortOrder]
            Sorts the returned list by when the card was created with the specified order.
            This field defaults to ASC.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Card]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.cards.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/cards",
            method="GET",
            params={
                "cursor": cursor,
                "customer_id": customer_id,
                "include_disabled": include_disabled,
                "reference_id": reference_id,
                "sort_order": sort_order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCardsResponse,
                    construct_type(
                        type_=ListCardsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    customer_id=customer_id,
                    include_disabled=include_disabled,
                    reference_id=reference_id,
                    sort_order=sort_order,
                    request_options=request_options,
                )
                _items = _parsed_response.cards
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        idempotency_key: str,
        source_id: str,
        card: CardParams,
        verification_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCardResponse:
        """
        Adds a card on file to an existing merchant.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this CreateCard request. Keys can be
            any valid string and must be unique for every request.

            Max: 45 characters

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        source_id : str
            The ID of the source which represents the card information to be stored. This can be a card nonce or a payment id.

        card : CardParams
            Payment details associated with the card to be stored.

        verification_token : typing.Optional[str]
            An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

            See the [SCA Overview](https://developer.squareup.com/docs/sca-overview).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.cards.create(
                idempotency_key="4935a656-a929-4792-b97c-8848be85c27c",
                source_id="cnon:uIbfJXhXETSP197M3GB",
                card={
                    "cardholder_name": "Amelia Earhart",
                    "billing_address": {
                        "address_line1": "500 Electric Ave",
                        "address_line2": "Suite 600",
                        "locality": "New York",
                        "administrative_district_level1": "NY",
                        "postal_code": "10003",
                        "country": "US",
                    },
                    "customer_id": "VDKXEEKPJN48QDG3BGGFAK05P8",
                    "reference_id": "user-id-1",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key,
            source_id=source_id,
            card=card,
            verification_token=verification_token,
            request_options=request_options,
        )
        return response.data

    async def get(self, card_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetCardResponse:
        """
        Retrieves details for a specific Card.

        Parameters
        ----------
        card_id : str
            Unique ID for the desired Card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.cards.get(
                card_id="card_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(card_id, request_options=request_options)
        return response.data

    async def disable(
        self, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DisableCardResponse:
        """
        Disables the card, preventing any further updates or charges.
        Disabling an already disabled card is allowed but has no effect.

        Parameters
        ----------
        card_id : str
            Unique ID for the desired Card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DisableCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.cards.disable(
                card_id="card_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.disable(card_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TaxIds(UncheckedBaseModel):
    """
    Identifiers for the location used by various governments for tax purposes.
    """

    eu_vat: typing.Optional[str] = pydantic.Field(default=None)
    """
    The EU VAT number for this location. For example, `IE3426675K`.
    If the EU VAT number is present, it is well-formed and has been
    validated with VIES, the VAT Information Exchange System.
    """

    fr_siret: typing.Optional[str] = pydantic.Field(default=None)
    """
    The SIRET (Système d'Identification du Répertoire des Entreprises et de leurs Etablissements)
    number is a 14-digit code issued by the French INSEE. For example, `39922799000021`.
    """

    fr_naf: typing.Optional[str] = pydantic.Field(default=None)
    """
    The French government uses the NAF (Nomenclature des Activités Françaises) to display and
    track economic statistical data. This is also called the APE (Activite Principale de l’Entreprise) code.
    For example, `6910Z`.
    """

    es_nif: typing.Optional[str] = pydantic.Field(default=None)
    """
    The NIF (Numero de Identificacion Fiscal) number is a nine-character tax identifier used in Spain.
    If it is present, it has been validated. For example, `73628495A`.
    """

    jp_qii: typing.Optional[str] = pydantic.Field(default=None)
    """
    The QII (Qualified Invoice Issuer) number is a 14-character tax identifier used in Japan.
    For example, `T1234567890123`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .catalog_item_food_and_beverage_details_dietary_preference_type import (
    CatalogItemFoodAndBeverageDetailsDietaryPreferenceType,
)
import pydantic
from .catalog_item_food_and_beverage_details_ingredient_standard_ingredient import (
    CatalogItemFoodAndBeverageDetailsIngredientStandardIngredient,
)
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogItemFoodAndBeverageDetailsIngredient(UncheckedBaseModel):
    """
    Describes the ingredient used in a `FOOD_AND_BEV` item.
    """

    type: typing.Optional[CatalogItemFoodAndBeverageDetailsDietaryPreferenceType] = pydantic.Field(default=None)
    """
    The dietary preference type of the ingredient. Supported values include `STANDARD` and `CUSTOM` as specified in `FoodAndBeverageDetails.DietaryPreferenceType`.
    See [DietaryPreferenceType](#type-dietarypreferencetype) for possible values
    """

    standard_name: typing.Optional[CatalogItemFoodAndBeverageDetailsIngredientStandardIngredient] = pydantic.Field(
        default=None
    )
    """
    The name of the ingredient from a standard pre-defined list. This should be null if it's a custom dietary preference.
    See [StandardIngredient](#type-standardingredient) for possible values
    """

    custom_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of a custom user-defined ingredient. This should be null if it's a standard dietary preference.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .search_availability_filter import SearchAvailabilityFilter
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class SearchAvailabilityQuery(UncheckedBaseModel):
    """
    The query used to search for buyer-accessible availabilities of bookings.
    """

    filter: SearchAvailabilityFilter = pydantic.Field()
    """
    The query filter to search for buyer-accessible availabilities of existing bookings.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .checkout_merchant_settings_payment_methods import CheckoutMerchantSettingsPaymentMethods
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CheckoutMerchantSettings(UncheckedBaseModel):
    payment_methods: typing.Optional[CheckoutMerchantSettingsPaymentMethods] = pydantic.Field(default=None)
    """
    The set of payment methods accepted for the merchant's account.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the settings were last updated, in RFC 3339 format.
    Examples for January 25th, 2020 6:25:34pm Pacific Standard Time:
    UTC: 2020-01-26T02:25:34Z
    Pacific Standard Time with UTC offset: 2020-01-25T18:25:34-08:00
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_account import LoyaltyAccountParams


class CreateLoyaltyAccountResponseParams(typing_extensions.TypedDict):
    """
    A response that includes loyalty account created.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    loyalty_account: typing_extensions.NotRequired[LoyaltyAccountParams]
    """
    The newly created loyalty account.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.gift_card_activity_clear_balance_reason import GiftCardActivityClearBalanceReason


class GiftCardActivityClearBalanceParams(typing_extensions.TypedDict):
    """
    Represents details about a `CLEAR_BALANCE` [gift card activity type](entity:GiftCardActivityType).
    """

    reason: GiftCardActivityClearBalanceReason
    """
    The reason the gift card balance was cleared.
    See [Reason](#type-reason) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawLocationsClient
from .custom_attribute_definitions.client import CustomAttributeDefinitionsClient
from .custom_attributes.client import CustomAttributesClient
from .transactions.client import TransactionsClient
from ..core.request_options import RequestOptions
from ..types.list_locations_response import ListLocationsResponse
from ..requests.location import LocationParams
from ..types.create_location_response import CreateLocationResponse
from ..types.get_location_response import GetLocationResponse
from ..types.update_location_response import UpdateLocationResponse
from ..requests.create_order_request import CreateOrderRequestParams
from ..requests.address import AddressParams
from ..requests.charge_request_additional_recipient import ChargeRequestAdditionalRecipientParams
from ..types.create_checkout_response import CreateCheckoutResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawLocationsClient
from .custom_attribute_definitions.client import AsyncCustomAttributeDefinitionsClient
from .custom_attributes.client import AsyncCustomAttributesClient
from .transactions.client import AsyncTransactionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class LocationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawLocationsClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = CustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.custom_attributes = CustomAttributesClient(client_wrapper=client_wrapper)

        self.transactions = TransactionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawLocationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawLocationsClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListLocationsResponse:
        """
        Provides details about all of the seller's [locations](https://developer.squareup.com/docs/locations-api),
        including those with an inactive status. Locations are listed alphabetically by `name`.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListLocationsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.list()
        """
        response = self._raw_client.list(request_options=request_options)
        return response.data

    def create(
        self,
        *,
        location: typing.Optional[LocationParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLocationResponse:
        """
        Creates a [location](https://developer.squareup.com/docs/locations-api).
        Creating new locations allows for separate configuration of receipt layouts, item prices,
        and sales reports. Developers can use locations to separate sales activity through applications
        that integrate with Square from sales activity elsewhere in a seller's account.
        Locations created programmatically with the Locations API last forever and
        are visible to the seller for their own management. Therefore, ensure that
        each location has a sensible and unique name.

        Parameters
        ----------
        location : typing.Optional[LocationParams]
            The initial values of the location being created. The `name` field is required and must be unique within a seller account.
            All other fields are optional, but any information you care about for the location should be included.
            The remaining fields are automatically added based on the data from the [main location](https://developer.squareup.com/docs/locations-api#about-the-main-location).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLocationResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.create(
            location={
                "name": "Midtown",
                "address": {
                    "address_line1": "1234 Peachtree St. NE",
                    "locality": "Atlanta",
                    "administrative_district_level1": "GA",
                    "postal_code": "30309",
                },
                "description": "Midtown Atlanta store",
            },
        )
        """
        response = self._raw_client.create(location=location, request_options=request_options)
        return response.data

    def get(self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetLocationResponse:
        """
        Retrieves details of a single location. Specify "main"
        as the location ID to retrieve details of the [main location](https://developer.squareup.com/docs/locations-api#about-the-main-location).

        Parameters
        ----------
        location_id : str
            The ID of the location to retrieve. Specify the string
            "main" to return the main location.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLocationResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.get(
            location_id="location_id",
        )
        """
        response = self._raw_client.get(location_id, request_options=request_options)
        return response.data

    def update(
        self,
        location_id: str,
        *,
        location: typing.Optional[LocationParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateLocationResponse:
        """
        Updates a [location](https://developer.squareup.com/docs/locations-api).

        Parameters
        ----------
        location_id : str
            The ID of the location to update.

        location : typing.Optional[LocationParams]
            The `Location` object with only the fields to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateLocationResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.update(
            location_id="location_id",
            location={
                "business_hours": {
                    "periods": [
                        {
                            "day_of_week": "FRI",
                            "start_local_time": "07:00",
                            "end_local_time": "18:00",
                        },
                        {
                            "day_of_week": "SAT",
                            "start_local_time": "07:00",
                            "end_local_time": "18:00",
                        },
                        {
                            "day_of_week": "SUN",
                            "start_local_time": "09:00",
                            "end_local_time": "15:00",
                        },
                    ]
                },
                "description": "Midtown Atlanta store - Open weekends",
            },
        )
        """
        response = self._raw_client.update(location_id, location=location, request_options=request_options)
        return response.data

    def checkouts(
        self,
        location_id: str,
        *,
        idempotency_key: str,
        order: CreateOrderRequestParams,
        ask_for_shipping_address: typing.Optional[bool] = OMIT,
        merchant_support_email: typing.Optional[str] = OMIT,
        pre_populate_buyer_email: typing.Optional[str] = OMIT,
        pre_populate_shipping_address: typing.Optional[AddressParams] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        additional_recipients: typing.Optional[typing.Sequence[ChargeRequestAdditionalRecipientParams]] = OMIT,
        note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCheckoutResponse:
        """
        Links a `checkoutId` to a `checkout_page_url` that customers are
        directed to in order to provide their payment information using a
        payment processing workflow hosted on connect.squareup.com.


        NOTE: The Checkout API has been updated with new features.
        For more information, see [Checkout API highlights](https://developer.squareup.com/docs/checkout-api#checkout-api-highlights).

        Parameters
        ----------
        location_id : str
            The ID of the business location to associate the checkout with.

        idempotency_key : str
            A unique string that identifies this checkout among others you have created. It can be
            any valid string but must be unique for every order sent to Square Checkout for a given location ID.

            The idempotency key is used to avoid processing the same order more than once. If you are
            unsure whether a particular checkout was created successfully, you can attempt it again with
            the same idempotency key and all the same other parameters without worrying about creating duplicates.

            You should use a random number/string generator native to the language
            you are working in to generate strings for your idempotency keys.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        order : CreateOrderRequestParams
            The order including line items to be checked out.

        ask_for_shipping_address : typing.Optional[bool]
            If `true`, Square Checkout collects shipping information on your behalf and stores
            that information with the transaction information in the Square Seller Dashboard.

            Default: `false`.

        merchant_support_email : typing.Optional[str]
            The email address to display on the Square Checkout confirmation page
            and confirmation email that the buyer can use to contact the seller.

            If this value is not set, the confirmation page and email display the
            primary email address associated with the seller's Square account.

            Default: none; only exists if explicitly set.

        pre_populate_buyer_email : typing.Optional[str]
            If provided, the buyer's email is prepopulated on the checkout page
            as an editable text field.

            Default: none; only exists if explicitly set.

        pre_populate_shipping_address : typing.Optional[AddressParams]
            If provided, the buyer's shipping information is prepopulated on the
            checkout page as editable text fields.

            Default: none; only exists if explicitly set.

        redirect_url : typing.Optional[str]
            The URL to redirect to after the checkout is completed with `checkoutId`,
            `transactionId`, and `referenceId` appended as URL parameters. For example,
            if the provided redirect URL is `http://www.example.com/order-complete`, a
            successful transaction redirects the customer to:

            `http://www.example.com/order-complete?checkoutId=xxxxxx&amp;referenceId=xxxxxx&amp;transactionId=xxxxxx`

            If you do not provide a redirect URL, Square Checkout displays an order
            confirmation page on your behalf; however, it is strongly recommended that
            you provide a redirect URL so you can verify the transaction results and
            finalize the order through your existing/normal confirmation workflow.

            Default: none; only exists if explicitly set.

        additional_recipients : typing.Optional[typing.Sequence[ChargeRequestAdditionalRecipientParams]]
            The basic primitive of a multi-party transaction. The value is optional.
            The transaction facilitated by you can be split from here.

            If you provide this value, the `amount_money` value in your `additional_recipients` field
            cannot be more than 90% of the `total_money` calculated by Square for your order.
            The `location_id` must be a valid seller location where the checkout is occurring.

            This field requires `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission.

            This field is currently not supported in the Square Sandbox.

        note : typing.Optional[str]
            An optional note to associate with the `checkout` object.

            This value cannot exceed 60 characters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCheckoutResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.checkouts(
            location_id="location_id",
            idempotency_key="86ae1696-b1e3-4328-af6d-f1e04d947ad6",
            order={
                "order": {
                    "location_id": "location_id",
                    "reference_id": "reference_id",
                    "customer_id": "customer_id",
                    "line_items": [
                        {
                            "name": "Printed T Shirt",
                            "quantity": "2",
                            "applied_taxes": [
                                {"tax_uid": "38ze1696-z1e3-5628-af6d-f1e04d947fg3"}
                            ],
                            "applied_discounts": [
                                {"discount_uid": "56ae1696-z1e3-9328-af6d-f1e04d947gd4"}
                            ],
                            "base_price_money": {"amount": 1500, "currency": "USD"},
                        },
                        {
                            "name": "Slim Jeans",
                            "quantity": "1",
                            "base_price_money": {"amount": 2500, "currency": "USD"},
                        },
                        {
                            "name": "Woven Sweater",
                            "quantity": "3",
                            "base_price_money": {"amount": 3500, "currency": "USD"},
                        },
                    ],
                    "taxes": [
                        {
                            "uid": "38ze1696-z1e3-5628-af6d-f1e04d947fg3",
                            "type": "INCLUSIVE",
                            "percentage": "7.75",
                            "scope": "LINE_ITEM",
                        }
                    ],
                    "discounts": [
                        {
                            "uid": "56ae1696-z1e3-9328-af6d-f1e04d947gd4",
                            "type": "FIXED_AMOUNT",
                            "amount_money": {"amount": 100, "currency": "USD"},
                            "scope": "LINE_ITEM",
                        }
                    ],
                },
                "idempotency_key": "12ae1696-z1e3-4328-af6d-f1e04d947gd4",
            },
            ask_for_shipping_address=True,
            merchant_support_email="merchant+support@website.com",
            pre_populate_buyer_email="example@email.com",
            pre_populate_shipping_address={
                "address_line1": "1455 Market St.",
                "address_line2": "Suite 600",
                "locality": "San Francisco",
                "administrative_district_level1": "CA",
                "postal_code": "94103",
                "country": "US",
                "first_name": "Jane",
                "last_name": "Doe",
            },
            redirect_url="https://merchant.website.com/order-confirm",
            additional_recipients=[
                {
                    "location_id": "057P5VYJ4A5X1",
                    "description": "Application fees",
                    "amount_money": {"amount": 60, "currency": "USD"},
                }
            ],
        )
        """
        response = self._raw_client.checkouts(
            location_id,
            idempotency_key=idempotency_key,
            order=order,
            ask_for_shipping_address=ask_for_shipping_address,
            merchant_support_email=merchant_support_email,
            pre_populate_buyer_email=pre_populate_buyer_email,
            pre_populate_shipping_address=pre_populate_shipping_address,
            redirect_url=redirect_url,
            additional_recipients=additional_recipients,
            note=note,
            request_options=request_options,
        )
        return response.data


class AsyncLocationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawLocationsClient(client_wrapper=client_wrapper)
        self.custom_attribute_definitions = AsyncCustomAttributeDefinitionsClient(client_wrapper=client_wrapper)

        self.custom_attributes = AsyncCustomAttributesClient(client_wrapper=client_wrapper)

        self.transactions = AsyncTransactionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawLocationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawLocationsClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListLocationsResponse:
        """
        Provides details about all of the seller's [locations](https://developer.squareup.com/docs/locations-api),
        including those with an inactive status. Locations are listed alphabetically by `name`.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListLocationsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.list()


        asyncio.run(main())
        """
        response = await self._raw_client.list(request_options=request_options)
        return response.data

    async def create(
        self,
        *,
        location: typing.Optional[LocationParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLocationResponse:
        """
        Creates a [location](https://developer.squareup.com/docs/locations-api).
        Creating new locations allows for separate configuration of receipt layouts, item prices,
        and sales reports. Developers can use locations to separate sales activity through applications
        that integrate with Square from sales activity elsewhere in a seller's account.
        Locations created programmatically with the Locations API last forever and
        are visible to the seller for their own management. Therefore, ensure that
        each location has a sensible and unique name.

        Parameters
        ----------
        location : typing.Optional[LocationParams]
            The initial values of the location being created. The `name` field is required and must be unique within a seller account.
            All other fields are optional, but any information you care about for the location should be included.
            The remaining fields are automatically added based on the data from the [main location](https://developer.squareup.com/docs/locations-api#about-the-main-location).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLocationResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.create(
                location={
                    "name": "Midtown",
                    "address": {
                        "address_line1": "1234 Peachtree St. NE",
                        "locality": "Atlanta",
                        "administrative_district_level1": "GA",
                        "postal_code": "30309",
                    },
                    "description": "Midtown Atlanta store",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(location=location, request_options=request_options)
        return response.data

    async def get(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLocationResponse:
        """
        Retrieves details of a single location. Specify "main"
        as the location ID to retrieve details of the [main location](https://developer.squareup.com/docs/locations-api#about-the-main-location).

        Parameters
        ----------
        location_id : str
            The ID of the location to retrieve. Specify the string
            "main" to return the main location.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLocationResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.get(
                location_id="location_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(location_id, request_options=request_options)
        return response.data

    async def update(
        self,
        location_id: str,
        *,
        location: typing.Optional[LocationParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateLocationResponse:
        """
        Updates a [location](https://developer.squareup.com/docs/locations-api).

        Parameters
        ----------
        location_id : str
            The ID of the location to update.

        location : typing.Optional[LocationParams]
            The `Location` object with only the fields to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateLocationResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.update(
                location_id="location_id",
                location={
                    "business_hours": {
                        "periods": [
                            {
                                "day_of_week": "FRI",
                                "start_local_time": "07:00",
                                "end_local_time": "18:00",
                            },
                            {
                                "day_of_week": "SAT",
                                "start_local_time": "07:00",
                                "end_local_time": "18:00",
                            },
                            {
                                "day_of_week": "SUN",
                                "start_local_time": "09:00",
                                "end_local_time": "15:00",
                            },
                        ]
                    },
                    "description": "Midtown Atlanta store - Open weekends",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(location_id, location=location, request_options=request_options)
        return response.data

    async def checkouts(
        self,
        location_id: str,
        *,
        idempotency_key: str,
        order: CreateOrderRequestParams,
        ask_for_shipping_address: typing.Optional[bool] = OMIT,
        merchant_support_email: typing.Optional[str] = OMIT,
        pre_populate_buyer_email: typing.Optional[str] = OMIT,
        pre_populate_shipping_address: typing.Optional[AddressParams] = OMIT,
        redirect_url: typing.Optional[str] = OMIT,
        additional_recipients: typing.Optional[typing.Sequence[ChargeRequestAdditionalRecipientParams]] = OMIT,
        note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCheckoutResponse:
        """
        Links a `checkoutId` to a `checkout_page_url` that customers are
        directed to in order to provide their payment information using a
        payment processing workflow hosted on connect.squareup.com.


        NOTE: The Checkout API has been updated with new features.
        For more information, see [Checkout API highlights](https://developer.squareup.com/docs/checkout-api#checkout-api-highlights).

        Parameters
        ----------
        location_id : str
            The ID of the business location to associate the checkout with.

        idempotency_key : str
            A unique string that identifies this checkout among others you have created. It can be
            any valid string but must be unique for every order sent to Square Checkout for a given location ID.

            The idempotency key is used to avoid processing the same order more than once. If you are
            unsure whether a particular checkout was created successfully, you can attempt it again with
            the same idempotency key and all the same other parameters without worrying about creating duplicates.

            You should use a random number/string generator native to the language
            you are working in to generate strings for your idempotency keys.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        order : CreateOrderRequestParams
            The order including line items to be checked out.

        ask_for_shipping_address : typing.Optional[bool]
            If `true`, Square Checkout collects shipping information on your behalf and stores
            that information with the transaction information in the Square Seller Dashboard.

            Default: `false`.

        merchant_support_email : typing.Optional[str]
            The email address to display on the Square Checkout confirmation page
            and confirmation email that the buyer can use to contact the seller.

            If this value is not set, the confirmation page and email display the
            primary email address associated with the seller's Square account.

            Default: none; only exists if explicitly set.

        pre_populate_buyer_email : typing.Optional[str]
            If provided, the buyer's email is prepopulated on the checkout page
            as an editable text field.

            Default: none; only exists if explicitly set.

        pre_populate_shipping_address : typing.Optional[AddressParams]
            If provided, the buyer's shipping information is prepopulated on the
            checkout page as editable text fields.

            Default: none; only exists if explicitly set.

        redirect_url : typing.Optional[str]
            The URL to redirect to after the checkout is completed with `checkoutId`,
            `transactionId`, and `referenceId` appended as URL parameters. For example,
            if the provided redirect URL is `http://www.example.com/order-complete`, a
            successful transaction redirects the customer to:

            `http://www.example.com/order-complete?checkoutId=xxxxxx&amp;referenceId=xxxxxx&amp;transactionId=xxxxxx`

            If you do not provide a redirect URL, Square Checkout displays an order
            confirmation page on your behalf; however, it is strongly recommended that
            you provide a redirect URL so you can verify the transaction results and
            finalize the order through your existing/normal confirmation workflow.

            Default: none; only exists if explicitly set.

        additional_recipients : typing.Optional[typing.Sequence[ChargeRequestAdditionalRecipientParams]]
            The basic primitive of a multi-party transaction. The value is optional.
            The transaction facilitated by you can be split from here.

            If you provide this value, the `amount_money` value in your `additional_recipients` field
            cannot be more than 90% of the `total_money` calculated by Square for your order.
            The `location_id` must be a valid seller location where the checkout is occurring.

            This field requires `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission.

            This field is currently not supported in the Square Sandbox.

        note : typing.Optional[str]
            An optional note to associate with the `checkout` object.

            This value cannot exceed 60 characters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCheckoutResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.checkouts(
                location_id="location_id",
                idempotency_key="86ae1696-b1e3-4328-af6d-f1e04d947ad6",
                order={
                    "order": {
                        "location_id": "location_id",
                        "reference_id": "reference_id",
                        "customer_id": "customer_id",
                        "line_items": [
                            {
                                "name": "Printed T Shirt",
                                "quantity": "2",
                                "applied_taxes": [
                                    {"tax_uid": "38ze1696-z1e3-5628-af6d-f1e04d947fg3"}
                                ],
                                "applied_discounts": [
                                    {
                                        "discount_uid": "56ae1696-z1e3-9328-af6d-f1e04d947gd4"
                                    }
                                ],
                                "base_price_money": {"amount": 1500, "currency": "USD"},
                            },
                            {
                                "name": "Slim Jeans",
                                "quantity": "1",
                                "base_price_money": {"amount": 2500, "currency": "USD"},
                            },
                            {
                                "name": "Woven Sweater",
                                "quantity": "3",
                                "base_price_money": {"amount": 3500, "currency": "USD"},
                            },
                        ],
                        "taxes": [
                            {
                                "uid": "38ze1696-z1e3-5628-af6d-f1e04d947fg3",
                                "type": "INCLUSIVE",
                                "percentage": "7.75",
                                "scope": "LINE_ITEM",
                            }
                        ],
                        "discounts": [
                            {
                                "uid": "56ae1696-z1e3-9328-af6d-f1e04d947gd4",
                                "type": "FIXED_AMOUNT",
                                "amount_money": {"amount": 100, "currency": "USD"},
                                "scope": "LINE_ITEM",
                            }
                        ],
                    },
                    "idempotency_key": "12ae1696-z1e3-4328-af6d-f1e04d947gd4",
                },
                ask_for_shipping_address=True,
                merchant_support_email="merchant+support@website.com",
                pre_populate_buyer_email="example@email.com",
                pre_populate_shipping_address={
                    "address_line1": "1455 Market St.",
                    "address_line2": "Suite 600",
                    "locality": "San Francisco",
                    "administrative_district_level1": "CA",
                    "postal_code": "94103",
                    "country": "US",
                    "first_name": "Jane",
                    "last_name": "Doe",
                },
                redirect_url="https://merchant.website.com/order-confirm",
                additional_recipients=[
                    {
                        "location_id": "057P5VYJ4A5X1",
                        "description": "Application fees",
                        "amount_money": {"amount": 60, "currency": "USD"},
                    }
                ],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.checkouts(
            location_id,
            idempotency_key=idempotency_key,
            order=order,
            ask_for_shipping_address=ask_for_shipping_address,
            merchant_support_email=merchant_support_email,
            pre_populate_buyer_email=pre_populate_buyer_email,
            pre_populate_shipping_address=pre_populate_shipping_address,
            redirect_url=redirect_url,
            additional_recipients=additional_recipients,
            note=note,
            request_options=request_options,
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .search_orders_state_filter import SearchOrdersStateFilter
import pydantic
from .search_orders_date_time_filter import SearchOrdersDateTimeFilter
from .search_orders_fulfillment_filter import SearchOrdersFulfillmentFilter
from .search_orders_source_filter import SearchOrdersSourceFilter
from .search_orders_customer_filter import SearchOrdersCustomerFilter
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchOrdersFilter(UncheckedBaseModel):
    """
    Filtering criteria to use for a `SearchOrders` request. Multiple filters
    are ANDed together.
    """

    state_filter: typing.Optional[SearchOrdersStateFilter] = pydantic.Field(default=None)
    """
    Filter by [OrderState](entity:OrderState).
    """

    date_time_filter: typing.Optional[SearchOrdersDateTimeFilter] = pydantic.Field(default=None)
    """
    Filter for results within a time range.
    
    __Important:__ If you filter for orders by time range, you must set `SearchOrdersSort`
    to sort by the same field.
    [Learn more about filtering orders by time range.](https://developer.squareup.com/docs/orders-api/manage-orders/search-orders#important-note-about-filtering-orders-by-time-range)
    """

    fulfillment_filter: typing.Optional[SearchOrdersFulfillmentFilter] = pydantic.Field(default=None)
    """
    Filter by the fulfillment type or state.
    """

    source_filter: typing.Optional[SearchOrdersSourceFilter] = pydantic.Field(default=None)
    """
    Filter by the source of the order.
    """

    customer_filter: typing.Optional[SearchOrdersCustomerFilter] = pydantic.Field(default=None)
    """
    Filter by customers associated with the order.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .customer_creation_source_filter import CustomerCreationSourceFilter
import pydantic
from .time_range import TimeRange
from .customer_text_filter import CustomerTextFilter
from .filter_value import FilterValue
from .customer_custom_attribute_filters import CustomerCustomAttributeFilters
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerFilter(UncheckedBaseModel):
    """
    Represents the filtering criteria in a [search query](entity:CustomerQuery) that defines how to filter
    customer profiles returned in [SearchCustomers](api-endpoint:Customers-SearchCustomers) results.
    """

    creation_source: typing.Optional[CustomerCreationSourceFilter] = pydantic.Field(default=None)
    """
    A filter to select customers based on their creation source.
    """

    created_at: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    A filter to select customers based on when they were created.
    """

    updated_at: typing.Optional[TimeRange] = pydantic.Field(default=None)
    """
    A filter to select customers based on when they were last updated.
    """

    email_address: typing.Optional[CustomerTextFilter] = pydantic.Field(default=None)
    """
    A filter to [select customers by their email address](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#search-by-email-address) 
    visible to the seller. 
    This filter is case-insensitive.
    
    For [exact matching](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#exact-search-by-email-address), this
    filter causes the search to return customer profiles 
    whose `email_address` field value are identical to the email address provided
    in the query.
    
    For [fuzzy matching](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#fuzzy-search-by-email-address), 
    this filter causes the search to return customer profiles 
    whose `email_address` field value has a token-wise partial match against the filtering 
    expression in the query. For example, with `Steven gmail` provided in a search
    query, the search returns customers whose email address is `steven.johnson@gmail.com` 
    or `mygmail@stevensbakery.com`. Square removes any punctuation (including periods (.),
    underscores (_), and the @ symbol) and tokenizes the email addresses on spaces. A match is
    found if a tokenized email address contains all the tokens in the search query, 
    irrespective of the token order.
    """

    phone_number: typing.Optional[CustomerTextFilter] = pydantic.Field(default=None)
    """
    A filter to [select customers by their phone numbers](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#search-by-phone-number)
    visible to the seller.
    
    For [exact matching](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#exact-search-by-phone-number),
    this filter returns customers whose phone number matches the specified query expression. The number in the query must be of an
    E.164-compliant form. In particular, it must include the leading `+` sign followed by a country code and then a subscriber number.
    For example, the standard E.164 form of a US phone number is `+12062223333` and an E.164-compliant variation is `+1 (206) 222-3333`.
    To match the query expression, stored customer phone numbers are converted to the standard E.164 form.
    
    For [fuzzy matching](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#fuzzy-search-by-phone-number),
    this filter returns customers whose phone number matches the token or tokens provided in the query expression. For example, with `415`
    provided in a search query, the search returns customers with the phone numbers `+1-415-212-1200`, `+1-212-415-1234`, and `+1 (551) 234-1567`.
    Similarly, a search query of `415 123` returns customers with the phone numbers `+1-212-415-1234` and `+1 (551) 234-1567` but not
    `+1-212-415-1200`. A match is found if a tokenized phone number contains all the tokens in the search query, irrespective of the token order.
    """

    reference_id: typing.Optional[CustomerTextFilter] = pydantic.Field(default=None)
    """
    A filter to [select customers by their reference IDs](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#search-by-reference-id).
    This filter is case-insensitive.
    
    [Exact matching](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#exact-search-by-reference-id) 
    of a customer's reference ID against a query's reference ID is evaluated as an
    exact match between two strings, character by character in the given order.
    
    [Fuzzy matching](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#fuzzy-search-by-reference-id) 
    of stored reference IDs against queried reference IDs works 
    exactly the same as fuzzy matching on email addresses. Non-alphanumeric characters 
    are replaced by spaces to tokenize stored and queried reference IDs. A match is found
    if a tokenized stored reference ID contains all tokens specified in any order in the query. For example,
    a query of `NYC M` matches customer profiles with the `reference_id` value of `NYC_M_35_JOHNSON`
    and `NYC_27_MURRAY`.
    """

    group_ids: typing.Optional[FilterValue] = pydantic.Field(default=None)
    """
    A filter to select customers based on the [groups](entity:CustomerGroup) they belong to.
    Group membership is controlled by sellers and developers.
    
    The `group_ids` filter has the following syntax:
    ```
    "group_ids": {
    "any":  ["{group_a_id}", "{group_b_id}", ...],
    "all":  ["{group_1_id}", "{group_2_id}", ...],
    "none": ["{group_i_id}", "{group_ii_id}", ...]
    }
    ```
    
    You can use any combination of the `any`, `all`, and `none` fields in the filter.
    With `any`, the search returns customers in groups `a` or `b` or any other group specified in the list.
    With `all`, the search returns customers in groups `1` and `2` and all other groups specified in the list.
    With `none`, the search returns customers not in groups `i` or `ii` or any other group specified in the list.
    
    If any of the search conditions are not met, including when an invalid or non-existent group ID is provided,
    the result is an empty object (`{}`).
    """

    custom_attribute: typing.Optional[CustomerCustomAttributeFilters] = pydantic.Field(default=None)
    """
    A filter to select customers based on one or more custom attributes.  
    This filter can contain up to 10 custom attribute filters. Each custom attribute filter specifies filtering criteria for a target custom
    attribute. If multiple custom attribute filters are provided, they are combined as an `AND` operation.
    
    To be valid for a search, the custom attributes must be visible to the requesting application. For more information, including example queries,
    see [Search by custom attribute](https://developer.squareup.com/docs/customers-api/use-the-api/search-customers#search-by-custom-attribute).
    
    Square returns matching customer profiles, which do not contain custom attributes. To retrieve customer-related custom attributes,
    use the [Customer Custom Attributes API](api:CustomerCustomAttributes). For example, you can call
    [RetrieveCustomerCustomAttribute](api-endpoint:CustomerCustomAttributes-RetrieveCustomerCustomAttribute) using a customer ID from the result set.
    """

    segment_ids: typing.Optional[FilterValue] = pydantic.Field(default=None)
    """
     A filter to select customers based on the [segments](entity:CustomerSegment) they belong to.
    Segment membership is dynamic and adjusts automatically based on whether customers meet the segment criteria.
    
    You can provide up to three segment IDs in the filter, using any combination of the `all`, `any`, and `none` fields.
    For the following example, the results include customers who belong to both segment A and segment B but do not belong to segment C.
    
    ```
    "segment_ids": {
    "all":  ["{segment_A_id}", "{segment_B_id}"],
    "none":  ["{segment_C_id}"]
    }
    ```
    
    If an invalid or non-existent segment ID is provided in the filter, Square stops processing the request
    and returns a `400 BAD_REQUEST` error that includes the segment ID.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .catalog_modifier import CatalogModifier
import pydantic
from .catalog_custom_attribute_value import CatalogCustomAttributeValue
import typing_extensions
from .catalog_v1id import CatalogV1Id
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogObjectModifier(UncheckedBaseModel):
    type: typing.Literal["MODIFIER"] = "MODIFIER"
    modifier_data: typing.Optional[CatalogModifier] = pydantic.Field(default=None)
    """
    Structured data for a `CatalogModifier`, set for CatalogObjects of type `MODIFIER`.
    """

    id: str = pydantic.Field()
    """
    An identifier to reference this object in the catalog. When a new `CatalogObject`
    is inserted, the client should set the id to a temporary identifier starting with
    a "`#`" character. Other objects being inserted or updated within the same request
    may use this identifier to refer to the new object.
    
    When the server receives the new object, it will supply a unique identifier that
    replaces the temporary identifier for all future references.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    Last modification [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates) in RFC 3339 format, e.g., `"2016-08-15T23:59:33.123Z"`
    would indicate the UTC time (denoted by `Z`) of August 15, 2016 at 23:59:33 and 123 milliseconds.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version of the object. When updating an object, the version supplied
    must match the version in the database, otherwise the write will be rejected as conflicting.
    """

    is_deleted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If `true`, the object has been deleted from the database. Must be `false` for new objects
    being inserted. When deleted, the `updated_at` field will equal the deletion time.
    """

    custom_attribute_values: typing.Optional[typing.Dict[str, CatalogCustomAttributeValue]] = pydantic.Field(
        default=None
    )
    """
    A map (key-value pairs) of application-defined custom attribute values. The value of a key-value pair
    is a [CatalogCustomAttributeValue](entity:CatalogCustomAttributeValue) object. The key is the `key` attribute
    value defined in the associated [CatalogCustomAttributeDefinition](entity:CatalogCustomAttributeDefinition)
    object defined by the application making the request.
    
    If the `CatalogCustomAttributeDefinition` object is
    defined by another application, the `CatalogCustomAttributeDefinition`'s key attribute value is prefixed by
    the defining application ID. For example, if the `CatalogCustomAttributeDefinition` has a `key` attribute of
    `"cocoa_brand"` and the defining application ID is `"abcd1234"`, the key in the map is `"abcd1234:cocoa_brand"`
    if the application making the request is different from the application defining the custom attribute definition.
    Otherwise, the key used in the map is simply `"cocoa_brand"`.
    
    Application-defined custom attributes are set at a global (location-independent) level.
    Custom attribute values are intended to store additional information about a catalog object
    or associations with an entity in another system. Do not use custom attributes
    to store any sensitive information (personally identifiable information, card details, etc.).
    """

    catalog_v1ids: typing_extensions.Annotated[
        typing.Optional[typing.List[CatalogV1Id]], FieldMetadata(alias="catalog_v1_ids")
    ] = pydantic.Field(default=None)
    """
    The Connect v1 IDs for this object at each location where it is present, where they
    differ from the object's Connect V2 ID. The field will only be present for objects that
    have been created or modified by legacy APIs.
    """

    present_at_all_locations: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If `true`, this object is present at all locations (including future locations), except where specified in
    the `absent_at_location_ids` field. If `false`, this object is not present at any locations (including future locations),
    except where specified in the `present_at_location_ids` field. If not specified, defaults to `true`.
    """

    present_at_location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of locations where the object is present, even if `present_at_all_locations` is `false`.
    This can include locations that are deactivated.
    """

    absent_at_location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of locations where the object is not present, even if `present_at_all_locations` is `true`.
    This can include locations that are deactivated.
    """

    image_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Identifies the `CatalogImage` attached to this `CatalogObject`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .team_member import TeamMember
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class UpdateTeamMemberRequest(UncheckedBaseModel):
    """
    Represents an update request for a `TeamMember` object.
    """

    team_member: typing.Optional[TeamMember] = pydantic.Field(default=None)
    """
    The team member fields to add, change, or clear. Fields can be cleared using a null value. To update
    `wage_setting.job_assignments`, you must provide the complete list of job assignments. If needed, call
    [ListJobs](api-endpoint:Team-ListJobs) to get the required `job_id` values.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyPromotionAvailableTimeData(UncheckedBaseModel):
    """
    Represents scheduling information that determines when purchases can qualify to earn points
    from a [loyalty promotion](entity:LoyaltyPromotion).
    """

    start_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The date that the promotion starts, in `YYYY-MM-DD` format. Square populates this field
    based on the provided `time_periods`.
    """

    end_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The date that the promotion ends, in `YYYY-MM-DD` format. Square populates this field
    based on the provided `time_periods`. If an end date is not specified, an `ACTIVE` promotion
    remains available until it is canceled.
    """

    time_periods: typing.List[str] = pydantic.Field()
    """
    A list of [iCalendar (RFC 5545) events](https://tools.ietf.org/html/rfc5545#section-3.6.1)
    (`VEVENT`). Each event represents an available time period per day or days of the week. 
    A day can have a maximum of one available time period.
    
    Only `DTSTART`, `DURATION`, and `RRULE` are supported. `DTSTART` and `DURATION` are required and
    timestamps must be in local (unzoned) time format. Include `RRULE` to specify recurring promotions,
    an end date (using the `UNTIL` keyword), or both. For more information, see
    [Available time](https://developer.squareup.com/docs/loyalty-api/loyalty-promotions#available-time).
    
    Note that `BEGIN:VEVENT` and `END:VEVENT` are optional in a `CreateLoyaltyPromotion` request
    but are always included in the response.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

MeasurementUnitLength = typing.Union[
    typing.Literal[
        "IMPERIAL_INCH",
        "IMPERIAL_FOOT",
        "IMPERIAL_YARD",
        "IMPERIAL_MILE",
        "METRIC_MILLIMETER",
        "METRIC_CENTIMETER",
        "METRIC_METER",
        "METRIC_KILOMETER",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawTeamMembersClient
from .wage_setting.client import WageSettingClient
from ..requests.team_member import TeamMemberParams
from ..core.request_options import RequestOptions
from ..types.create_team_member_response import CreateTeamMemberResponse
from ..requests.create_team_member_request import CreateTeamMemberRequestParams
from ..types.batch_create_team_members_response import BatchCreateTeamMembersResponse
from ..requests.update_team_member_request import UpdateTeamMemberRequestParams
from ..types.batch_update_team_members_response import BatchUpdateTeamMembersResponse
from ..requests.search_team_members_query import SearchTeamMembersQueryParams
from ..types.search_team_members_response import SearchTeamMembersResponse
from ..types.get_team_member_response import GetTeamMemberResponse
from ..types.update_team_member_response import UpdateTeamMemberResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawTeamMembersClient
from .wage_setting.client import AsyncWageSettingClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TeamMembersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTeamMembersClient(client_wrapper=client_wrapper)
        self.wage_setting = WageSettingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTeamMembersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTeamMembersClient
        """
        return self._raw_client

    def create(
        self,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        team_member: typing.Optional[TeamMemberParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTeamMemberResponse:
        """
        Creates a single `TeamMember` object. The `TeamMember` object is returned on successful creates.
        You must provide the following values in your request to this endpoint:
        - `given_name`
        - `family_name`

        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#createteammember).

        Parameters
        ----------
        idempotency_key : typing.Optional[str]
            A unique string that identifies this `CreateTeamMember` request.
            Keys can be any valid string, but must be unique for every request.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

            The minimum length is 1 and the maximum length is 45.

        team_member : typing.Optional[TeamMemberParams]
            **Required** The data used to create the `TeamMember` object. If you include `wage_setting`, you must provide
            `job_id` for each job assignment. To get job IDs, call [ListJobs](api-endpoint:Team-ListJobs).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTeamMemberResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team_members.create(
            idempotency_key="idempotency-key-0",
            team_member={
                "reference_id": "reference_id_1",
                "status": "ACTIVE",
                "given_name": "Joe",
                "family_name": "Doe",
                "email_address": "joe_doe@gmail.com",
                "phone_number": "+14159283333",
                "assigned_locations": {
                    "assignment_type": "EXPLICIT_LOCATIONS",
                    "location_ids": ["YSGH2WBKG94QZ", "GA2Y9HSJ8KRYT"],
                },
                "wage_setting": {
                    "job_assignments": [
                        {
                            "pay_type": "SALARY",
                            "annual_rate": {"amount": 3000000, "currency": "USD"},
                            "weekly_hours": 40,
                            "job_id": "FjS8x95cqHiMenw4f1NAUH4P",
                        },
                        {
                            "pay_type": "HOURLY",
                            "hourly_rate": {"amount": 2000, "currency": "USD"},
                            "job_id": "VDNpRv8da51NU8qZFC5zDWpF",
                        },
                    ],
                    "is_overtime_exempt": True,
                },
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key, team_member=team_member, request_options=request_options
        )
        return response.data

    def batch_create(
        self,
        *,
        team_members: typing.Dict[str, CreateTeamMemberRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchCreateTeamMembersResponse:
        """
        Creates multiple `TeamMember` objects. The created `TeamMember` objects are returned on successful creates.
        This process is non-transactional and processes as much of the request as possible. If one of the creates in
        the request cannot be successfully processed, the request is not marked as failed, but the body of the response
        contains explicit error information for the failed create.

        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#bulk-create-team-members).

        Parameters
        ----------
        team_members : typing.Dict[str, CreateTeamMemberRequestParams]
            The data used to create the `TeamMember` objects. Each key is the `idempotency_key` that maps to the `CreateTeamMemberRequest`.
            The maximum number of create objects is 25.

            If you include a team member's `wage_setting`, you must provide `job_id` for each job assignment. To get job IDs,
            call [ListJobs](api-endpoint:Team-ListJobs).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCreateTeamMembersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team_members.batch_create(
            team_members={
                "idempotency-key-1": {
                    "team_member": {
                        "reference_id": "reference_id_1",
                        "given_name": "Joe",
                        "family_name": "Doe",
                        "email_address": "joe_doe@gmail.com",
                        "phone_number": "+14159283333",
                        "assigned_locations": {
                            "assignment_type": "EXPLICIT_LOCATIONS",
                            "location_ids": ["YSGH2WBKG94QZ", "GA2Y9HSJ8KRYT"],
                        },
                    }
                },
                "idempotency-key-2": {
                    "team_member": {
                        "reference_id": "reference_id_2",
                        "given_name": "Jane",
                        "family_name": "Smith",
                        "email_address": "jane_smith@gmail.com",
                        "phone_number": "+14159223334",
                        "assigned_locations": {
                            "assignment_type": "ALL_CURRENT_AND_FUTURE_LOCATIONS"
                        },
                    }
                },
            },
        )
        """
        response = self._raw_client.batch_create(team_members=team_members, request_options=request_options)
        return response.data

    def batch_update(
        self,
        *,
        team_members: typing.Dict[str, UpdateTeamMemberRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchUpdateTeamMembersResponse:
        """
        Updates multiple `TeamMember` objects. The updated `TeamMember` objects are returned on successful updates.
        This process is non-transactional and processes as much of the request as possible. If one of the updates in
        the request cannot be successfully processed, the request is not marked as failed, but the body of the response
        contains explicit error information for the failed update.
        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#bulk-update-team-members).

        Parameters
        ----------
        team_members : typing.Dict[str, UpdateTeamMemberRequestParams]
            The data used to update the `TeamMember` objects. Each key is the `team_member_id` that maps to the `UpdateTeamMemberRequest`.
            The maximum number of update objects is 25.

            For each team member, include the fields to add, change, or clear. Fields can be cleared using a null value.
            To update `wage_setting.job_assignments`, you must provide the complete list of job assignments. If needed,
            call [ListJobs](api-endpoint:Team-ListJobs) to get the required `job_id` values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchUpdateTeamMembersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team_members.batch_update(
            team_members={
                "AFMwA08kR-MIF-3Vs0OE": {
                    "team_member": {
                        "reference_id": "reference_id_2",
                        "is_owner": False,
                        "status": "ACTIVE",
                        "given_name": "Jane",
                        "family_name": "Smith",
                        "email_address": "jane_smith@gmail.com",
                        "phone_number": "+14159223334",
                        "assigned_locations": {
                            "assignment_type": "ALL_CURRENT_AND_FUTURE_LOCATIONS"
                        },
                    }
                },
                "fpgteZNMaf0qOK-a4t6P": {
                    "team_member": {
                        "reference_id": "reference_id_1",
                        "is_owner": False,
                        "status": "ACTIVE",
                        "given_name": "Joe",
                        "family_name": "Doe",
                        "email_address": "joe_doe@gmail.com",
                        "phone_number": "+14159283333",
                        "assigned_locations": {
                            "assignment_type": "EXPLICIT_LOCATIONS",
                            "location_ids": ["YSGH2WBKG94QZ", "GA2Y9HSJ8KRYT"],
                        },
                    }
                },
            },
        )
        """
        response = self._raw_client.batch_update(team_members=team_members, request_options=request_options)
        return response.data

    def search(
        self,
        *,
        query: typing.Optional[SearchTeamMembersQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchTeamMembersResponse:
        """
        Returns a paginated list of `TeamMember` objects for a business.
        The list can be filtered by location IDs, `ACTIVE` or `INACTIVE` status, or whether
        the team member is the Square account owner.

        Parameters
        ----------
        query : typing.Optional[SearchTeamMembersQueryParams]
            The query parameters.

        limit : typing.Optional[int]
            The maximum number of `TeamMember` objects in a page (100 by default).

        cursor : typing.Optional[str]
            The opaque cursor for fetching the next page. For more information, see
            [pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchTeamMembersResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team_members.search(
            query={"filter": {"location_ids": ["0G5P3VGACMMQZ"], "status": "ACTIVE"}},
            limit=10,
        )
        """
        response = self._raw_client.search(query=query, limit=limit, cursor=cursor, request_options=request_options)
        return response.data

    def get(
        self, team_member_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTeamMemberResponse:
        """
        Retrieves a `TeamMember` object for the given `TeamMember.id`.
        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#retrieve-a-team-member).

        Parameters
        ----------
        team_member_id : str
            The ID of the team member to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTeamMemberResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team_members.get(
            team_member_id="team_member_id",
        )
        """
        response = self._raw_client.get(team_member_id, request_options=request_options)
        return response.data

    def update(
        self,
        team_member_id: str,
        *,
        team_member: typing.Optional[TeamMemberParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateTeamMemberResponse:
        """
        Updates a single `TeamMember` object. The `TeamMember` object is returned on successful updates.
        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#update-a-team-member).

        Parameters
        ----------
        team_member_id : str
            The ID of the team member to update.

        team_member : typing.Optional[TeamMemberParams]
            The team member fields to add, change, or clear. Fields can be cleared using a null value. To update
            `wage_setting.job_assignments`, you must provide the complete list of job assignments. If needed, call
            [ListJobs](api-endpoint:Team-ListJobs) to get the required `job_id` values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateTeamMemberResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team_members.update(
            team_member_id="team_member_id",
            team_member={
                "reference_id": "reference_id_1",
                "status": "ACTIVE",
                "given_name": "Joe",
                "family_name": "Doe",
                "email_address": "joe_doe@gmail.com",
                "phone_number": "+14159283333",
                "assigned_locations": {
                    "assignment_type": "EXPLICIT_LOCATIONS",
                    "location_ids": ["YSGH2WBKG94QZ", "GA2Y9HSJ8KRYT"],
                },
                "wage_setting": {
                    "job_assignments": [
                        {
                            "pay_type": "SALARY",
                            "annual_rate": {"amount": 3000000, "currency": "USD"},
                            "weekly_hours": 40,
                            "job_id": "FjS8x95cqHiMenw4f1NAUH4P",
                        },
                        {
                            "pay_type": "HOURLY",
                            "hourly_rate": {"amount": 1200, "currency": "USD"},
                            "job_id": "VDNpRv8da51NU8qZFC5zDWpF",
                        },
                    ],
                    "is_overtime_exempt": True,
                },
            },
        )
        """
        response = self._raw_client.update(team_member_id, team_member=team_member, request_options=request_options)
        return response.data


class AsyncTeamMembersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTeamMembersClient(client_wrapper=client_wrapper)
        self.wage_setting = AsyncWageSettingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTeamMembersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTeamMembersClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        team_member: typing.Optional[TeamMemberParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTeamMemberResponse:
        """
        Creates a single `TeamMember` object. The `TeamMember` object is returned on successful creates.
        You must provide the following values in your request to this endpoint:
        - `given_name`
        - `family_name`

        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#createteammember).

        Parameters
        ----------
        idempotency_key : typing.Optional[str]
            A unique string that identifies this `CreateTeamMember` request.
            Keys can be any valid string, but must be unique for every request.
            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

            The minimum length is 1 and the maximum length is 45.

        team_member : typing.Optional[TeamMemberParams]
            **Required** The data used to create the `TeamMember` object. If you include `wage_setting`, you must provide
            `job_id` for each job assignment. To get job IDs, call [ListJobs](api-endpoint:Team-ListJobs).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTeamMemberResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team_members.create(
                idempotency_key="idempotency-key-0",
                team_member={
                    "reference_id": "reference_id_1",
                    "status": "ACTIVE",
                    "given_name": "Joe",
                    "family_name": "Doe",
                    "email_address": "joe_doe@gmail.com",
                    "phone_number": "+14159283333",
                    "assigned_locations": {
                        "assignment_type": "EXPLICIT_LOCATIONS",
                        "location_ids": ["YSGH2WBKG94QZ", "GA2Y9HSJ8KRYT"],
                    },
                    "wage_setting": {
                        "job_assignments": [
                            {
                                "pay_type": "SALARY",
                                "annual_rate": {"amount": 3000000, "currency": "USD"},
                                "weekly_hours": 40,
                                "job_id": "FjS8x95cqHiMenw4f1NAUH4P",
                            },
                            {
                                "pay_type": "HOURLY",
                                "hourly_rate": {"amount": 2000, "currency": "USD"},
                                "job_id": "VDNpRv8da51NU8qZFC5zDWpF",
                            },
                        ],
                        "is_overtime_exempt": True,
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key, team_member=team_member, request_options=request_options
        )
        return response.data

    async def batch_create(
        self,
        *,
        team_members: typing.Dict[str, CreateTeamMemberRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchCreateTeamMembersResponse:
        """
        Creates multiple `TeamMember` objects. The created `TeamMember` objects are returned on successful creates.
        This process is non-transactional and processes as much of the request as possible. If one of the creates in
        the request cannot be successfully processed, the request is not marked as failed, but the body of the response
        contains explicit error information for the failed create.

        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#bulk-create-team-members).

        Parameters
        ----------
        team_members : typing.Dict[str, CreateTeamMemberRequestParams]
            The data used to create the `TeamMember` objects. Each key is the `idempotency_key` that maps to the `CreateTeamMemberRequest`.
            The maximum number of create objects is 25.

            If you include a team member's `wage_setting`, you must provide `job_id` for each job assignment. To get job IDs,
            call [ListJobs](api-endpoint:Team-ListJobs).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCreateTeamMembersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team_members.batch_create(
                team_members={
                    "idempotency-key-1": {
                        "team_member": {
                            "reference_id": "reference_id_1",
                            "given_name": "Joe",
                            "family_name": "Doe",
                            "email_address": "joe_doe@gmail.com",
                            "phone_number": "+14159283333",
                            "assigned_locations": {
                                "assignment_type": "EXPLICIT_LOCATIONS",
                                "location_ids": ["YSGH2WBKG94QZ", "GA2Y9HSJ8KRYT"],
                            },
                        }
                    },
                    "idempotency-key-2": {
                        "team_member": {
                            "reference_id": "reference_id_2",
                            "given_name": "Jane",
                            "family_name": "Smith",
                            "email_address": "jane_smith@gmail.com",
                            "phone_number": "+14159223334",
                            "assigned_locations": {
                                "assignment_type": "ALL_CURRENT_AND_FUTURE_LOCATIONS"
                            },
                        }
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_create(team_members=team_members, request_options=request_options)
        return response.data

    async def batch_update(
        self,
        *,
        team_members: typing.Dict[str, UpdateTeamMemberRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchUpdateTeamMembersResponse:
        """
        Updates multiple `TeamMember` objects. The updated `TeamMember` objects are returned on successful updates.
        This process is non-transactional and processes as much of the request as possible. If one of the updates in
        the request cannot be successfully processed, the request is not marked as failed, but the body of the response
        contains explicit error information for the failed update.
        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#bulk-update-team-members).

        Parameters
        ----------
        team_members : typing.Dict[str, UpdateTeamMemberRequestParams]
            The data used to update the `TeamMember` objects. Each key is the `team_member_id` that maps to the `UpdateTeamMemberRequest`.
            The maximum number of update objects is 25.

            For each team member, include the fields to add, change, or clear. Fields can be cleared using a null value.
            To update `wage_setting.job_assignments`, you must provide the complete list of job assignments. If needed,
            call [ListJobs](api-endpoint:Team-ListJobs) to get the required `job_id` values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchUpdateTeamMembersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team_members.batch_update(
                team_members={
                    "AFMwA08kR-MIF-3Vs0OE": {
                        "team_member": {
                            "reference_id": "reference_id_2",
                            "is_owner": False,
                            "status": "ACTIVE",
                            "given_name": "Jane",
                            "family_name": "Smith",
                            "email_address": "jane_smith@gmail.com",
                            "phone_number": "+14159223334",
                            "assigned_locations": {
                                "assignment_type": "ALL_CURRENT_AND_FUTURE_LOCATIONS"
                            },
                        }
                    },
                    "fpgteZNMaf0qOK-a4t6P": {
                        "team_member": {
                            "reference_id": "reference_id_1",
                            "is_owner": False,
                            "status": "ACTIVE",
                            "given_name": "Joe",
                            "family_name": "Doe",
                            "email_address": "joe_doe@gmail.com",
                            "phone_number": "+14159283333",
                            "assigned_locations": {
                                "assignment_type": "EXPLICIT_LOCATIONS",
                                "location_ids": ["YSGH2WBKG94QZ", "GA2Y9HSJ8KRYT"],
                            },
                        }
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_update(team_members=team_members, request_options=request_options)
        return response.data

    async def search(
        self,
        *,
        query: typing.Optional[SearchTeamMembersQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchTeamMembersResponse:
        """
        Returns a paginated list of `TeamMember` objects for a business.
        The list can be filtered by location IDs, `ACTIVE` or `INACTIVE` status, or whether
        the team member is the Square account owner.

        Parameters
        ----------
        query : typing.Optional[SearchTeamMembersQueryParams]
            The query parameters.

        limit : typing.Optional[int]
            The maximum number of `TeamMember` objects in a page (100 by default).

        cursor : typing.Optional[str]
            The opaque cursor for fetching the next page. For more information, see
            [pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchTeamMembersResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team_members.search(
                query={
                    "filter": {"location_ids": ["0G5P3VGACMMQZ"], "status": "ACTIVE"}
                },
                limit=10,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            query=query, limit=limit, cursor=cursor, request_options=request_options
        )
        return response.data

    async def get(
        self, team_member_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTeamMemberResponse:
        """
        Retrieves a `TeamMember` object for the given `TeamMember.id`.
        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#retrieve-a-team-member).

        Parameters
        ----------
        team_member_id : str
            The ID of the team member to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTeamMemberResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team_members.get(
                team_member_id="team_member_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(team_member_id, request_options=request_options)
        return response.data

    async def update(
        self,
        team_member_id: str,
        *,
        team_member: typing.Optional[TeamMemberParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateTeamMemberResponse:
        """
        Updates a single `TeamMember` object. The `TeamMember` object is returned on successful updates.
        Learn about [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#update-a-team-member).

        Parameters
        ----------
        team_member_id : str
            The ID of the team member to update.

        team_member : typing.Optional[TeamMemberParams]
            The team member fields to add, change, or clear. Fields can be cleared using a null value. To update
            `wage_setting.job_assignments`, you must provide the complete list of job assignments. If needed, call
            [ListJobs](api-endpoint:Team-ListJobs) to get the required `job_id` values.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateTeamMemberResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team_members.update(
                team_member_id="team_member_id",
                team_member={
                    "reference_id": "reference_id_1",
                    "status": "ACTIVE",
                    "given_name": "Joe",
                    "family_name": "Doe",
                    "email_address": "joe_doe@gmail.com",
                    "phone_number": "+14159283333",
                    "assigned_locations": {
                        "assignment_type": "EXPLICIT_LOCATIONS",
                        "location_ids": ["YSGH2WBKG94QZ", "GA2Y9HSJ8KRYT"],
                    },
                    "wage_setting": {
                        "job_assignments": [
                            {
                                "pay_type": "SALARY",
                                "annual_rate": {"amount": 3000000, "currency": "USD"},
                                "weekly_hours": 40,
                                "job_id": "FjS8x95cqHiMenw4f1NAUH4P",
                            },
                            {
                                "pay_type": "HOURLY",
                                "hourly_rate": {"amount": 1200, "currency": "USD"},
                                "job_id": "VDNpRv8da51NU8qZFC5zDWpF",
                            },
                        ],
                        "is_overtime_exempt": True,
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            team_member_id, team_member=team_member, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .vendor import VendorParams


class UpdateVendorRequestParams(typing_extensions.TypedDict):
    """
    Represents an input to a call to [UpdateVendor](api-endpoint:Vendors-UpdateVendor).
    """

    idempotency_key: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A client-supplied, universally unique identifier (UUID) for the
    request.
    
    See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
    [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
    information.
    """

    vendor: VendorParams
    """
    The specified [Vendor](entity:Vendor) to be updated.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .gift_card import GiftCardParams


class CreateGiftCardResponseParams(typing_extensions.TypedDict):
    """
    A response that contains a `GiftCard`. The response might contain a set of `Error` objects if the request
    resulted in errors.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    gift_card: typing_extensions.NotRequired[GiftCardParams]
    """
    The new gift card.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class TimeRangeParams(typing_extensions.TypedDict):
    """
    Represents a generic time range. The start and end values are
    represented in RFC 3339 format. Time ranges are customized to be
    inclusive or exclusive based on the needs of a particular endpoint.
    Refer to the relevant endpoint-specific documentation to determine
    how time ranges are handled.
    """

    start_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A datetime value in RFC 3339 format indicating when the time range
    starts.
    """

    end_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A datetime value in RFC 3339 format indicating when the time range
    ends.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .employee import EmployeeParams
from .error import ErrorParams


class ListEmployeesResponseParams(typing_extensions.TypedDict):
    employees: typing_extensions.NotRequired[typing.Sequence[EmployeeParams]]
    cursor: typing_extensions.NotRequired[str]
    """
    The token to be used to retrieve the next page of results.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class BulkDeleteLocationCustomAttributesResponseLocationCustomAttributeDeleteResponseParams(
    typing_extensions.TypedDict
):
    """
    Represents an individual delete response in a [BulkDeleteLocationCustomAttributes](api-endpoint:LocationCustomAttributes-BulkDeleteLocationCustomAttributes)
    request.
    """

    location_id: typing_extensions.NotRequired[str]
    """
    The ID of the location associated with the custom attribute.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred while processing the individual LocationCustomAttributeDeleteRequest request
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_reward import LoyaltyRewardParams


class GetLoyaltyRewardResponseParams(typing_extensions.TypedDict):
    """
    A response that includes the loyalty reward.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    reward: typing_extensions.NotRequired[LoyaltyRewardParams]
    """
    The loyalty reward retrieved.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .cash_drawer_shift_state import CashDrawerShiftState
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CashDrawerShiftSummary(UncheckedBaseModel):
    """
    The summary of a closed cash drawer shift.
    This model contains only the money counted to start a cash drawer shift, counted
    at the end of the shift, and the amount that should be in the drawer at shift
    end based on summing all cash drawer shift events.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The shift unique ID.
    """

    state: typing.Optional[CashDrawerShiftState] = pydantic.Field(default=None)
    """
    The shift current state.
    See [CashDrawerShiftState](#type-cashdrawershiftstate) for possible values
    """

    opened_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The shift start time in ISO 8601 format.
    """

    ended_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The shift end time in ISO 8601 format.
    """

    closed_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The shift close time in ISO 8601 format.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    An employee free-text description of a cash drawer shift.
    """

    opened_cash_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money in the cash drawer at the start of the shift. This
    must be a positive amount.
    """

    expected_cash_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money that should be in the cash drawer at the end of the
    shift, based on the cash drawer events on the shift.
    The amount is correct if all shift employees accurately recorded their
    cash drawer shift events. Unrecorded events and events with the wrong amount
    result in an incorrect expected_cash_money amount that can be negative.
    """

    closed_cash_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money found in the cash drawer at the end of the shift by
    an auditing employee. The amount must be greater than or equal to zero.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The shift start time in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The shift updated at time in RFC 3339 format.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the location the cash drawer shift belongs to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CustomerCreationSource = typing.Union[
    typing.Literal[
        "OTHER",
        "APPOINTMENTS",
        "COUPON",
        "DELETION_RECOVERY",
        "DIRECTORY",
        "EGIFTING",
        "EMAIL_COLLECTION",
        "FEEDBACK",
        "IMPORT",
        "INVOICES",
        "LOYALTY",
        "MARKETING",
        "MERGE",
        "ONLINE_STORE",
        "INSTANT_PROFILE",
        "TERMINAL",
        "THIRD_PARTY",
        "THIRD_PARTY_IMPORT",
        "UNMERGE_RECOVERY",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .custom_attribute import CustomAttribute
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GetCustomerCustomAttributeResponse(UncheckedBaseModel):
    """
    Represents a [RetrieveCustomerCustomAttribute](api-endpoint:CustomerCustomAttributes-RetrieveCustomerCustomAttribute) response.
    Either `custom_attribute_definition` or `errors` is present in the response.
    """

    custom_attribute: typing.Optional[CustomAttribute] = pydantic.Field(default=None)
    """
    The retrieved custom attribute. If `with_definition` was set to `true` in the request,
    the custom attribute definition is returned in the `definition` field.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .payment_refund import PaymentRefundParams


class ListPaymentRefundsResponseParams(typing_extensions.TypedDict):
    """
    Defines the response returned by [ListPaymentRefunds](api-endpoint:Refunds-ListPaymentRefunds).

    Either `errors` or `refunds` is present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    refunds: typing_extensions.NotRequired[typing.Sequence[PaymentRefundParams]]
    """
    The list of requested refunds.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If empty,
    this is the final response.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """
